@@ -196,8 +196,6 @@ private:
     Mat_<int> nfeatures_;
     Mat_<int> colors_;
     Mat_<float> weights_;
-
-    Mat buf_;
 };
 
 
@@ -428,8 +426,13 @@ void BackgroundSubtractorGMGImpl::apply(InputArray _frame, OutputArray _fgmask,
 {
     Mat frame = _frame.getMat();
 
-    CV_Assert(frame.depth() == CV_8U || frame.depth() == CV_16U || frame.depth() == CV_32F);
-    CV_Assert(frame.channels() == 1 || frame.channels() == 3 || frame.channels() == 4);
+    const int depth = frame.depth();
+    CV_CheckDepth(depth, (depth == CV_8U)  || (depth == CV_8S)  ||
+                         (depth == CV_16U) || (depth == CV_16S) ||
+                                              (depth == CV_32S) ||
+                         (depth == CV_32F) || (depth == CV_64F), "Unsupported depth");
+    CV_CheckGE(frame.channels(), 1, "Unsupported channels");
+    CV_CheckLE(frame.channels(), 4, "Unsupported channels");
 
     if (newLearningRate != -1.0)
     {
@@ -443,8 +446,12 @@ void BackgroundSubtractorGMGImpl::apply(InputArray _frame, OutputArray _fgmask,
         double maxval = maxVal_;
         if( minVal_ == 0 && maxVal_ == 0 )
         {
-            minval = 0;
-            maxval = frame.depth() == CV_8U ? 255.0 : frame.depth() == CV_16U ? std::numeric_limits<ushort>::max() : 1.0;
+            if( depth == CV_8U )        { minval = std::numeric_limits<uint8_t>::min(); maxval = std::numeric_limits<uint8_t>::max(); }
+            else if( depth == CV_8S )   { minval = std::numeric_limits<int8_t>::min();  maxval = std::numeric_limits<int8_t>::max();  }
+            else if( depth == CV_16U )  { minval = std::numeric_limits<uint16_t>::min();maxval = std::numeric_limits<uint16_t>::max();}
+            else if( depth == CV_16S )  { minval = std::numeric_limits<int16_t>::min(); maxval = std::numeric_limits<int16_t>::max(); }
+            else if( depth == CV_32S )  { minval = std::numeric_limits<int32_t>::min(); maxval = std::numeric_limits<int32_t>::max(); }
+            else /* CV_32F or CV_64F */ { minval = 0.0; maxval = 1.0; }
         }
         initialize(frame.size(), minval, maxval);
     }
@@ -459,8 +466,7 @@ void BackgroundSubtractorGMGImpl::apply(InputArray _frame, OutputArray _fgmask,
 
     if (smoothingRadius > 0)
     {
-        medianBlur(fgmask, buf_, smoothingRadius);
-        swap(fgmask, buf_);
+        medianBlur(fgmask, fgmask, smoothingRadius);
     }
 
     // keep track of how many frames we have processed
@@ -474,7 +480,6 @@ void BackgroundSubtractorGMGImpl::release()
     nfeatures_.release();
     colors_.release();
     weights_.release();
-    buf_.release();
 }
 
 
