@@ -1,11 +1,11 @@
+// This file is part of OpenCV project.
+// It is subject to the license terms in the LICENSE file found in the top-level directory
+// of this distribution and at http://opencv.org/license.html.
 #include "perf_precomp.hpp"
 
-using namespace std;
-using namespace cv;
-using namespace cv::xfeatures2d;
-using namespace perf;
-using std::tr1::make_tuple;
-using std::tr1::get;
+namespace opencv_test { namespace {
+
+#ifdef OPENCV_XFEATURES2D_HAS_VGG_DATA
 
 typedef perf::TestBaseWithParam<std::string> vgg;
 
@@ -33,3 +33,7 @@ PERF_TEST_P(vgg, extract, testing::Values(VGG_IMAGES))
 
     SANITY_CHECK_NOTHING();
 }
+
+#endif  // OPENCV_XFEATURES2D_HAS_VGG_DATA
+
+}} // namespace
